c  Program to integrate the 3D FITS-image over one axis.

c  Requires FITSIO library to compile.

      program  Integral
      implicit NONE
      character Name*64, OName*64
      integer NXmax,NYmax,NZmax, IX,IY,IZ, NP, Nout, K, L
      real Rpix(3), Delt(3), Rota(3)
      logical No_BUNIT, No_coord
      real*8 Rval(3)
      integer BlockSize, Bitpix, Naxis, Naxes(4), NA
      character Type(3)*8, BUNIT*8

      parameter (NXmax = 222)
      parameter (NYmax = 222) 
      parameter (NZmax = 123)

      real Array(NXmax,NYmax,NZmax), Out(NXmax,NYmax,NZmax)
      integer IArray(NXmax,NYmax,NZmax), IOut(NXmax,NYmax,NZmax)
      equivalence (Array,IArray), (Out,IOut)

      write (*,*) ' Name of input FITS-file:'
      read (*,'(A)')  Name
      write (*,*) Name
      write (*,*) ' Name of output FITS-file:'
      read (*,'(A)')  OName
      write (*,*) OName
      write (*,*) ' Number of the axis to sum along:'
      read (*,*) NA
      write (*,*) NA
      write (*,*) ' Number of consequtive planes to sum',
     #            ' (if 0 - all of them)'
      read (*,*) NP
      write (*,*) NP 

      if (NA.lt.1 .or. NA.gt.3)  stop

      call Reading_input_image (Name,Naxis,Naxes,No_BUNIT,BUNIT,
     #         Type,Rval,Rpix,Delt,Rota,No_coord,
     #         Array,NXmax,NYmax,NZmax, BlockSize,Bitpix)

      if (NP .eq. 0)  NP = Naxes(NA)

      Nout = Naxes(NA)/NP

      if (NA .eq. 3)  then

         do IX=1,Naxes(1)
            do IY=1,Naxes(2)
               L = 0
               do IZ=1,Nout
                  Out(IX,IY,IZ) = 0.0
                  do K=1,NP
                     L = L + 1
                     if (IOut(IX,IY,IZ) .ne. -1)  then
                        if (IArray(IX,IY,IZ) .ne. -1)  then
                          Out(IX,IY,IZ) =Out(IX,IY,IZ)+Array(IX,IY,L)/NP
                         else
                          IOut(IX,IY,IZ) = -1
                         end if
                        end if
                     end do
                  end do
               end do
            end do

      else
      if (NA .eq. 2)  then

         do IX=1,Naxes(1)
            do IZ=1,Naxes(3)
               L = 0
               do IY=1,Nout
                  Out(IX,IY,IZ) = 0.0
                  do K=1,NP
                     L = L + 1
                     if (IOut(IX,IY,IZ) .ne. -1)  then
                        if (IArray(IX,IY,IZ) .ne. -1)  then
                          Out(IX,IY,IZ) =Out(IX,IY,IZ)+Array(IX,L,IZ)/NP
                         else
                          IOut(IX,IY,IZ) = -1
                         end if
                        end if
                     end do
                  end do
               end do
            end do
      
      else

         do IY=1,Naxes(2)
            do IZ=1,Naxes(3)
               L = 0
               do IX=1,Nout
                  Out(IX,IY,IZ) = 0.0
                  do K=1,NP
                     L = L + 1
                     if (IOut(IX,IY,IZ) .ne. -1)  then
                        if (IArray(IX,IY,IZ) .ne. -1)  then
                          Out(IX,IY,IZ) =Out(IX,IY,IZ)+Array(L,IY,IZ)/NP
                         else
                          IOut(IX,IY,IZ) = -1
                         end if
                        end if
                     end do
                  end do
               end do
            end do
      end if
      end if

      Naxes(NA) = Nout
      Delt(NA) = Delt(NA) * NP
c !!!! Corrected bug:
      Rpix(NA) = (2*Rpix(NA)-1+NP) / float(2*NP)

      call Writing_output_image(Out,NXmax,NYmax,NZmax,
     #         OName, BlockSize, Naxes, BUNIT,No_BUNIT, Type,Rval,
     #         Rpix,Delt,Rota,No_coord,Bitpix)


      stop
      end


c=============================================================================


      subroutine Reading_input_image (Name,Naxis,Naxes,No_BUNIT,BUNIT,
     #         Type,Rval,Rpix,Delt,Rota,No_coord,
     #         Array,NXmax,NYmax,NZmax, BlockSize,Bitpix)

      implicit NONE
      character*(*) Name, Comment*1, BUNIT, Type(3), No*1
      integer BlockSize, Status, Bitpix, Naxis, Naxes(4), Pcount,Gcount,
     #        NXmax,NYmax,NZmax, I,IX,IY,IV, Blank
      real*8 Rval(3)
      real Rpix(3), Delt(3), Rota(3), Array(NXmax,NYmax,NZmax), Nullval
      logical Simple, Extend, No_BUNIT, No_coord, Anyf
      equivalence (Nullval,Blank)
     
      No_BUNIT = .false.
      No_coord = .false.
      Status = 0
      
      call FTopen (1, Name, 0, BlockSize, Status)
      if (Status .ne. 0) then
         write (*,*) Status
         write (*,'(A,A)') ' !!! Mistake in opening the file ', Name
         stop
         end if

      call FTghpr (1, 3, Simple, Bitpix, Naxis, Naxes, Pcount, Gcount,
     #            Extend, Status)
      if (Status .ne. 0)  then
         write (*,*) Status
         write (*,*) ' !!! Mistake in reading the header in ', Name
         stop
         end if

      if (Naxes(1).gt.NXmax.or.Naxes(2).gt.NYmax.or.Naxes(3).gt.NZmax)
     #     then
         write (*,*) ' Input image is bigger than',NXmax,'x',NYmax,'x',
     #               NZmax,' !!!'
         stop
         end if

      call FTgkyS (1, 'BUNIT', BUNIT, Comment, Status)
      if (Status .ne. 0)  then
         Status = 0
         No_BUNIT = .true.
         end if

      do I=1,3
         write (No,'(I1)') I
         call FTgkyS (1, 'CTYPE'//No//'  ', Type(I), Comment, Status)
         call FTgkyD (1, 'CRVAL'//No//'  ', Rval(I), Comment, Status)
         call FTgkyE (1, 'CDELT'//No//'  ', Delt(I), Comment, Status)
         call FTgkyE (1, 'CRPIX'//No//'  ', Rpix(I), Comment, Status)
c         call FTgkyE (1, 'CROTA'//No//'  ', Rota(I), Comment, Status)
         end do
      if (Status .ne. 0)  then
         Status = 0
         No_coord = .true.
         end if

      do IX=1,NXmax
         do IY=1,NYmax
            do IV=1,NZmax
               Array(IX,IY,IV) = 0.0
               end do
            end do
         end do

c  "Magic blanking" of undefined pixels: we use a Real representation
c  of Integer -1 as a magic value:
      Blank = -1

c  Reading the data array into Array:
      call FTg3dE (1,0,Nullval, NXmax,NYmax, Naxes(1),Naxes(2),Naxes(3), 
     #             Array, Anyf, Status)

      if (Anyf)  then
         write (*,*) ' !!! Undefined values in the image !'
c         stop
         end if
      if (Status .ne. 0)  then
         write (*,*) ' !!! Mistake in reading the file !'
         stop
         end if

      call FTclos (1, Status)

      return
      end
c=============================================================================


      subroutine Writing_output_image(Out,NXOmax,NYOmax,NZOmax,
     #         OName, BlockSize, Naxes, BUNIT,No_BUNIT, Type,Rval,
     #         Rpix,Delt,Rota,No_coord,Bitpix)

      implicit NONE
      real Data_max,Data_min
      integer IX,IY,IZ,NXOmax,NYOmax,NZOmax,BlockSize,Naxes(4),
     #        Status, Bitpix, I
      real Out(NXOmax,NYOmax,NZOmax), Rpix(3), Delt(3), Rota(3)
      real*8 Rval(3)
      character*(*) OName, BUNIT, Type(3), No*1
      logical Simple, Extend, No_BUNIT, No_coord

      Status = 0

c  Finding the extreme values in the enlarged image:
      Data_max = -1.e33   
      Data_min = 1.e33
      do IX=1,Naxes(1)
         do IY=1,Naxes(2)
            do IZ=1,Naxes(3)
               if (Out(IX,IY,IZ) .gt. Data_max)  Data_max=Out(IX,IY,IZ)
               if (Out(IX,IY,IZ) .lt. Data_min)  Data_min=Out(IX,IY,IZ)
               end do
            end do
         end do

c  Writing the output (bigger) FITS file:
      open (1, file=OName)
      close (1, status='DELETE')
      call FTinit (1, OName,Blocksize, Status)

      if (Status .ne. 0)  then
         write (*,*) '!!! Cannot create file', OName
         write (*,*) ' Status = ', Status
         stop
         end if

      Simple = .true.
      Extend = .false. 
c  Writing the primary keywords:
c !!! Works just for -32 format!
      call FTphpr (1, Simple, -32, 3, Naxes, 0, 1, Extend, Status)   
      if (Status .ne. 0)  stop

c  Writing the additional keywords:
      call FTpkyL (1, 'BLOCKED ', .true., ' ', Status)
      if (.not. No_BUNIT)  then
         call FTpkyS (1, 'BUNIT   ', BUNIT, ' ', Status)
         end if
      call FTpkyE (1, 'DATAMAX ', Data_max, 10, ' ', Status)
      call FTpkyE (1, 'DATAMIN ', Data_min, 10, ' ', Status)

      if (.not. No_coord)  then
          do I=1,3
             write (No,'(I1)') I
             call FTpkyS (1, 'CTYPE'//No//'  ', Type(I), ' ', Status)
             call FTpkyD (1, 'CRVAL'//No//'  ', Rval(I), 10, ' ',Status)
             call FTpkyE (1, 'CDELT'//No//'  ', Delt(I), 10, ' ',Status)
             call FTpkyE (1, 'CRPIX'//No//'  ', Rpix(I), 10, ' ',Status)
c             call FTpkyE (1, 'CROTA'//No//'  ', Rota(I), 10, ' ',Status)
             end do
          end if
         
      call FTpdat (1, Status)

      if (Status .ne. 0)  then
         write (*,*) '!!! Wrong header parameter'
         stop
         end if

c Writing the image array:

      call FTp3dE (1,1, NXOmax,NYOmax, Naxes(1),Naxes(2),Naxes(3),
     #            Out, Status)

      if (Status .ne. 0)  then
         write (*,*) '!!! Error while writing the data array'
         stop
         end if

      call FTclos (1, Status)

      return
      end
c=============================================================================

